@@ -16,10 +16,11 @@ module Spreadsheet
 class Reader
   include Spreadsheet::Encodings
   include Spreadsheet::Excel::Internals
-  ROW_BLOCK_OPS = [
-    :blank, :boolerr, :dbcell, :formula, :label, :labelsst, :mulblank, :mulrk,
-    :number, :rk, :rstring,
-  ]
+  ROW_BLOCK_OPS = {
+    :blank  => true, :boolerr  => true, :dbcell   => true, :formula => true,
+    :label  => true, :labelsst => true, :mulblank => true, :mulrk   => true,
+    :number => true, :rk       => true, :rstring  => true,
+  }
   def initialize opts = {}
     @pos = 0
     @bigendian = opts.fetch(:bigendian) {
@@ -378,6 +379,10 @@ class Reader
       formula.value = value
     elsif rtype == 0
       pos, op, len, work = get_next_chunk
+      if op == :sharedfmla
+        ## TODO: formula-support in 0.8.0
+        pos, op, len, work = get_next_chunk
+      end
       if op == :string
         formula.value = client read_string(work, 2), @workbook.encoding
       else
@@ -487,7 +492,7 @@ class Reader
         size, = work.unpack "x#{pos}V"
         pos += 4
         data = work[pos, size].chomp "\000\000"
-        link.url = client data, 'UTF-16LE'
+        link.url = client data
         pos += size
       else
         # 6.53.3 Hyperlink to a Local File
@@ -532,7 +537,7 @@ class Reader
         total, size = work.unpack "x#{pos}V2"
         pos += 10
         if total > 0
-          link.url = client work[pos, size], 'UTF-16LE'
+          link.url = client work[pos, size]
           pos += size
         end
       end
@@ -770,9 +775,9 @@ class Reader
       when :datemode   # ○  DATEMODE ➜ 6.25
         flag, _ = work.unpack 'v'
         if flag == 1
-          @workbook.date_base = Date.new 1904, 1, 1
+          @workbook.date_base = DateTime.new 1904, 1, 1
         else
-          @workbook.date_base = Date.new 1899, 12, 31
+          @workbook.date_base = DateTime.new 1899, 12, 31
         end
       when :continue   # ○  CONTINUE ➜ 6.22
         case previous_op
@@ -804,6 +809,7 @@ class Reader
   end
   def read_worksheet worksheet, offset
     @pos = offset
+    @detected_rows = {}
     previous = nil
     while tuple = get_next_chunk
       pos, op, len, work = tuple
@@ -835,6 +841,10 @@ class Reader
         read_hlink worksheet, work, pos, len
       when :window2
         read_window2 worksheet, work, pos, len
+      else
+        if ROW_BLOCK_OPS.include?(op)
+          set_missing_row_address worksheet, work, pos, len
+        end
       end
       previous = op
     end
@@ -1021,6 +1031,21 @@ class Reader
     cells.formats[column] = @workbook.format(xf) unless xf == 0
     cells[column] = value
   end
+  def set_missing_row_address worksheet, work, pos, len
+    # Offset  Size  Contents
+    #      0     2  Index of this row
+    #      2     2  Index to this column
+    row_index, column_index = work.unpack 'v2'
+    unless worksheet.offsets[row_index]
+      @current_row_block_offset ||= [pos]
+      data = {
+        :index          => row_index,
+        :row_block      => @current_row_block_offset,
+        :offset         => @current_row_block_offset[0],
+      }
+      worksheet.set_row_address row_index, data
+    end
+  end
   def set_row_address worksheet, work, pos, len
     # Offset  Size  Contents
     #      0     2  Index of this row
@@ -1095,9 +1120,18 @@ class Reader
     worksheet.set_row_address index, attrs
   end
   def setup io
+    ## Reading from StringIO fails without forced encoding
+    if io.respond_to?(:string) && (str = io.string) \
+      && str.respond_to?(:force_encoding)
+      str.force_encoding 'ASCII-8BIT'
+    end
+    ##
+    io.rewind
     @ole = Ole::Storage.open io
     @workbook = Workbook.new io, {}
-    @book = @ole.file.open("Book") rescue @ole.file.open("Workbook")
+    %w{Book Workbook BOOK WORKBOOK book workbook}.any? do |name|
+      @book = @ole.file.open(name) rescue false
+    end
     @data = @book.read
     read_bof
     @workbook.ole = @book
